"
COPYRIGHT (c) 2021 LabWare

              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libcompat' }"

"{ NameSpace: Smalltalk }"

ProjectDefinition subclass:#PackageManifest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'System-Support-Projects'
!

!PackageManifest class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2021 LabWare

              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    PackageManifest is a superclass for package meta-data classes used in Pharo
    world. This class is provided to support interoperability with Pharo (to limited
    extent).

    [author:]
        Jan Vrany <jan.vrany@labware.com>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!PackageManifest class methodsFor:'instance creation'!

newNamed:newName package:packageID
    | newClass |

    newClass := super newNamed:newName package:packageID.
    newClass category: packageID , '-Manifest'.
    ^ newClass

    "Created: / 26-11-2021 / 12:44:55 / Jan Vrany <jan.vrany@labware.com>"
! !

!PackageManifest class methodsFor:'accessing'!

projectDefinitionClassNameForDefinitionOf:aPackageId
    "given a package-ID, return an appropriate class name for this package"

    ^ 'Manifest' , (aPackageId select: [:c | c isLetterOrDigit ])

    "
    ProjectDefinition initialClassNameForDefinitionOf:'MachineArithmetic'
    PackageManifest initialClassNameForDefinitionOf:'MachineArithmetic'
    PackageManifest initialClassNameForDefinitionOf:'MachineArithmetic-FFI-SmalltalkX'
    "

    "Created: / 25-11-2021 / 14:46:26 / Jan Vrany <jan.vrany@labware.com>"
! !

!PackageManifest class methodsFor:'accessing - monticello'!

monticelloName
    "Return name of the package for Monticello / Tonel."

    ^ self package

    "Created: / 26-11-2021 / 12:42:25 / Jan Vrany <jan.vrany@labware.com>"
! !

!PackageManifest class methodsFor:'code generation'!

forEachContentsMethodsCodeToCompileDo:aTwoArgBlock ignoreOldDefinition:ignoreOldDefinition
    | cls |

    aTwoArgBlock
        value: self mandatoryPreRequisites_code
        value: 'stx - description'.

    aTwoArgBlock
        value: self referencedPreRequisites_code
        value: 'stx - description'.

    (self class includesSelector:#excludedFromPreRequisites) ifTrue:[
        aTwoArgBlock
            value: self excludedFromPreRequisites_code
            value: 'stx - description'.
    ].

    "Add additional custom files"
    cls := self.
    [ cls ~~ Object ] whileTrue:[
        cls class selectorsAndMethodsDo:[:selector :method |
            | annotation overwrite |

            annotation := method annotationAt: #method:category:overwrite:.
            annotation notNil ifTrue:[ 
                overwrite := annotation argumentAt: 3.
            ] ifFalse:[ 
                annotation := method annotationAt: #method:category:.
                overwrite := true.
            ].
            (annotation notNil and:[ overwrite or:[ cls class methodDictionary includesKey: (annotation argumentAt:1) ] ]) ifTrue:[
                | code |

                code := self perform: selector.
                code notNil ifTrue:[ 
                    aTwoArgBlock value: code value: 'stx - ' ,(annotation argumentAt: 2)
                ].
            ].
        ].
        cls := cls superclass.
    ].

    "Created: / 25-11-2021 / 16:10:43 / Jan Vrany <jan.vrany@labware.com>"
    "Modified: / 26-11-2021 / 12:29:26 / Jan Vrany <jan.vrany@labware.com>"
!

forEachDescriptionMethodsCodeToCompileDo:aTwoArgBlock ignoreOldDefinition:ignoreOldDefinition
    "Intentionally empty"

    "Created: / 25-11-2021 / 16:11:59 / Jan Vrany <jan.vrany@labware.com>"
! !

!PackageManifest class methodsFor:'description - classes'!

classNamesAndAttributes

    "/ For PackageManifests, we do not require #classNamesAndAttributes to
    "/ be explicitly specified. The reason is pragmatic - when a package uses
    "/ `PackageManifest`, most likely it is 'alien' Pharo package in which
    "/ package class list is not maintained.
    "/ 
    "/ Still, some ProjectDefinition APIs depend on it (for example, #testSuite)
    "/ so here we construct it dynamically.
    ^ self searchForClasses collect: [ :class | class name ] as: Array

    "
    ManifestMachineArithmetic classNamesAndAttributes
    "

    "Created: / 29-11-2021 / 22:37:37 / Jan Vrany <jan.vrany@labware.com>"
!

extensionMethodNames

    "/ For PackageManifests, we do not require #extensionMethodNames to
    "/ be explicitly specified. The reason is pragmatic - when a package uses
    "/ `PackageManifest`, most likely it is 'alien' Pharo package in which
    "/ package extension list is not maintained.
    "/ 
    "/ Still, some ProjectDefinition APIs depend on it (for example, #testSuite)
    "/ so here we construct it dynamically.
    | extensions extensionNames |

    extensions := self searchForExtensions.
    extensionNames := Array new: extensions size * 2.
    extensions withIndexDo: [ :m :i |
        extensionNames at: ((i - 1) * 2) + 1 put: m mclass name.
        extensionNames at: ((i - 1) * 2) + 2 put: m selector.
    ].
    ^ extensionNames

    "
    ManifestMachineArithmetic extensionMethodNames
    "

    "Created: / 29-11-2021 / 22:42:35 / Jan Vrany <jan.vrany@labware.com>"
! !

!PackageManifest class methodsFor:'private'!

setupForType:typeOrNil
    "Intentionally left empty"

    "Created: / 29-11-2021 / 13:51:14 / Jan Vrany <jan.vrany@labware.com>"
! !

!PackageManifest class methodsFor:'queries'!

isAbstract
    ^ self == PackageManifest

! !

!PackageManifest class methodsFor:'testing'!

isPackageManifest
    ^ self isAbstract not

    "
    PackageManifest isPackageManifest
    ManifestMachineArithmetic isPackageManifest
    "

    "Created: / 29-11-2021 / 14:44:57 / Jan Vrany <jan.vrany@labware.com>"
! !

!PackageManifest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

