"
 COPYRIGHT (c) 2006 by eXept Software AG
 COPYRIGHT (c) 2020-2022 LabWare
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libcompat' }"

"{ NameSpace: Smalltalk }"

LibraryDefinition subclass:#stx_libcompat
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* Projects & Packages *'
!

!stx_libcompat class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2006 by eXept Software AG
 COPYRIGHT (c) 2020-2022 LabWare
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    Package documentation

    This library contains compatibility classes,
    which are useful when loading code from other Smalltalk dialects.

    Some of the classes found here are simply empty, but mimic the class hierarchy of
    corresponding frameworks in Squeak, VisualWorks, Dolphin or VisualAge Smalltalk dialects.
    
    They usually only allow for an initial fileIn of such imported classes 
    (by giving them a superclass to be loaded).
    Importing then usually involves further rewrite and debugging - 
    especially if the imported code is UI related. 
    Non-UI code (model classes) often works out-of-the-box.

    [author:]
        cg

    [primary maintainer:]
        cg
"
! !

!stx_libcompat class methodsFor:'accessing - hg - settings'!

hgEnsureCopyrightMethod
    "If true, then #copyright method is automatically compiled in each class
     (but iff project definition defines it)

     Default is true (compile such method) but if the repository is mirror of CVS and
     you want to merge back to CVS at some point, you may want to not compile them
     to keep changes against CVS minimal"

    ^false "default"

    "Created: / 25-03-2021 / 14:30:04 / Jan Vrany <jan.vrany@labware.com>"
!

hgEnsureVersion_HGMethod
    "If true, then #version_HG method is automatically compiled in each class.

     Default is true (compile such method) but if the repository is mirror of CVS and
     you want to merge back to CVS at some point, you may want to not compile them
     to keep changes against CVS minimal. 

     If false, version_HG is compiled only in classes that has been modified
     and commited.

     Note that Mercurial can live without them
     just fine"

    ^false "default"

    "Created: / 25-03-2021 / 14:33:49 / Jan Vrany <jan.vrany@labware.com>"
!

hgRemoveContainesForDeletedClasses
    "If true, then containers for removed classes are __AUTOMATICALLY__ removed from the
     repositoru. If false, obsolete containes are kept.

     Default is true (remove obsolete containers) but if the repository is mirror of CVS and
     you want to merge back to CVS at some point, you may want to return false to avoid deletions
     of obsolete files. Usefull when branching off an old CVS repo with loads of mess."

    ^false

    "Created: / 25-03-2021 / 14:29:46 / Jan Vrany <jan.vrany@labware.com>"
! !

!stx_libcompat class methodsFor:'description'!

excludedFromPreRequisites
    "list all packages which should be ignored in the automatic
     preRequisites scan. See #preRequisites for more."

    ^ #(
        #'stx:goodies/monticello'    "PackageOrganizer - referenced by PackageInfo>>register "
        #'stx:libcomp'    "Parser - referenced by VAST::Application class>>initializeAfterLoad "
        #'stx:libtool'    "Tools::ClassCategoryList - referenced by SystemOrganization class>>addCategory: "
        #'stx:goodies/communication'    "HTTPInterface - referenced by HTTPSocket class>>httpGet:accept:" 
    )

    "Modified: / 22-09-2022 / 17:04:50 / Jan Vrany <jan.vrany@labware.com>"
!

mandatoryPreRequisites
    "list packages which are mandatory as a prerequisite.
     This are packages containing superclasses of my classes and classes which
     are extended by myself.
     They are mandatory, because we need these packages as a prerequisite for loading and compiling.
     This method is generated automatically,
     by searching along the inheritance chain of all of my classes.
     Please take a look at the #referencedPreRequisites method as well."

    ^ #(
        #'stx:libbasic'    "AbstractLock - superclass of Dolphin::Mutex"
        #'stx:libbasic2'    "PluggableDictionary - superclass of Dolphin::PluggableLookupTable"
        #'stx:libbasic3'    "ChangeSet - extended"
        #'stx:libview'    "Color - extended"
        #'stx:libview2'    "ApplicationModel - superclass of Dolphin::Shell"
        #'stx:libwidg'    "Button - superclass of LabeledBooleanView"
        #'stx:libwidg2'    "Slider - superclass of FractionalWidgetView"
    )
!

referencedPreRequisites
    "list packages which are a prerequisite, because they contain
     classes which are referenced by my classes.
     These packages are NOT needed as a prerequisite for compiling or loading,
     however, a class from it may be referenced during execution and having it
     unloaded then may lead to a runtime doesNotUnderstand error, unless the caller
     includes explicit checks for the package being present.
     This method is generated automatically,
     by searching all classes (and their packages) which are referenced by my classes.
     Please also take a look at the #mandatoryPreRequisites method"

    ^ #(
    )
!

subProjects
    "list packages which are known as subprojects.
     The generated makefile will enter those and make there as well.
     However: they are only built, not forced to be loaded when a package is loaded;
     for those, redefine #referencedPrerequisites or #mandatoryPreRequisites."

    ^ #(
    )
! !

!stx_libcompat class methodsFor:'description - actions'!

postLoadAction
    "invoked after loading a project"

    super postLoadAction.
    Smalltalk at:#Duration ifAbsentPut:TimeDuration.
    Smalltalk at:#KeyNotFound put:Object keyNotFoundSignal.

    "Modified: / 16-04-2021 / 14:13:09 / Jan Vrany <jan.vrany@labware.com>"
! !

!stx_libcompat class methodsFor:'description - compilation'!

additionalDefinitions_nt_dot_mak
    ^ '
# see stdHeader_nt for LIBCOMPAT_BASE
LIB_BASE_LD_ARG=-b:$(LIBCOMPAT_BASE)
'
!

stcWarningOptions
    ^ '-warnNonStandard -warnUnused'
! !

!stx_libcompat class methodsFor:'description - contents'!

classNamesAndAttributes
    "lists the classes which are to be included in the project.
     Each entry in the list may be: a single class-name (symbol),
     or an array-literal consisting of class name and attributes.
     Attributes are: #autoload or #<os> where os is one of win32, unix,..."

    ^ #(
        "<className> or (<className> attributes...) in load order"
        ActionButton
        ActionButtonView
        ActionMenu
        Alien
        Author
        BinaryStream
        BindingReference
        BorderDecorator
        CEnvironment
        CachedImage
        CharacterAttributes
        CharacterBlockScanner
        CodeController
        ColorForm
        ColorPreferencesCollection
        ComposedText
        ComposedTextView
        CompositeView
        CompositionScanner
        ConfigurationOf
        CoveragePalette
        CustomHelp
        DialogCompositeController
        DisplayScanner
        #'Dolphin::ClassCategoriesReader'
        #'Dolphin::DeafObject'
        #'Dolphin::Error'
        #'Dolphin::File'
        #'Dolphin::FileDialog'
        #'Dolphin::IXMLDOMDocument'
        #'Dolphin::IXMLDOMNamedNodeMap'
        #'Dolphin::ListPrompter'
        #'Dolphin::LookupTable'
        #'Dolphin::MessageBox'
        #'Dolphin::MethodCategoriesReader'
        #'Dolphin::Mutex'
        #'Dolphin::Notification'
        #'Dolphin::PluggableLookupTable'
        #'Dolphin::Prompter'
        #'Dolphin::SWEnvironment'
        #'Dolphin::SearchPolicy'
        #'Dolphin::SessionManager'
        #'Dolphin::SharedLookupTable'
        #'Dolphin::SharedSet'
        #'Dolphin::Shell'
        DragDropManager
        DragDropManagerTracker
        ErrorDiffusion
        ExternalReadStream
        ExternalWriteStream
        FileDirectory
        FileReference
        FractionalWidgetView
        GapString
        HTTPSocket
        IOAccessor
        ISO8859L1String
        InputState
        InstructionClient
        InstructionStream
        LabeledBooleanView
        ListItemWrapper
        MczInstaller
        MemoryPolicy
        MenuBuilder
        MinimalShellInterface
        MockMedium
        MouseMenuController
        MultiSelectionSequenceView
        NameSpaceOrganizer
        NetNameResolver
        NonInteractiveCompilerErrorHandler
        OpaqueImage
        OrderedDither
        PackageInfo
        PackageManifest
        ParagraphEditor
        PassivityWrapper
        Pixmap
        PluggableListView
        PluggableTextView
        Polyline
        Preferences
        ProtocolClient
        ProvideAnswerNotification
        RWBinaryOrTextStream
        RasterOp
        #'ReadMe::ST80_ReadMe'
        ScannerTable
        ScheduledControllers
        ScrollValueHolder
        ScrollWrapper
        SelectionInListController
        SelectionMenu
        SelectionTracker
        SmartRefStream
        SourceCodeStream
        #'Squeak::ClassCommentReader'
        #'Squeak::CustomHelp'
        #'Squeak::DebugAction'
        #'Squeak::DebugSession'
        #'Squeak::GLMCompositePresentation'
        #'Squeak::GTExecuteSelectionDebugAction'
        #'Squeak::GTGenericStackDebugger'
        #'Squeak::GTInspectSelectionDebugAction'
        #'Squeak::GTPrintSelectionDebugAction'
        #'Squeak::TextAnchor'
        #'Squeak::TextColor'
        #'Squeak::TextEmphasis'
        #'Squeak::TextFontChange'
        #'Squeak::TextURL'
        #'Squeak::VersionBrowser'
        StrikeFont
        StringCollationPolicy
        StringHolder
        SystemDictionary
        SystemOrganization
        SystemUtils
        SystemVersion
        TTFontDescription
        TextAttributes
        TextController
        TextLines
        TextList
        TextStyle
        TimeProfileBrowser
        TriggerButtonController
        TwoByteSymbol
        UILookPolicy
        UIManager
        UISettings
        UnixProcess
        Utilities
        #'V::ApplicationCoordinator'
        #'V::Dynalink32'
        #'V::File'
        #'V::GDIDLL'
        #'V::GraphPane'
        #'V::MainWindow'
        #'V::OLEControlSitePane'
        #'V::OLEStructure'
        #'V::ViewManager'
        #'VAST::CwDataInterchangeConverter'
        #'VAST::CwGlobalMemoryTransferMechanism'
        #'VAST::CwMainWindow'
        #'VAST::CwPrimitive'
        #'VAST::CwTransferMechanism'
        #'VAST::CwWidget'
        #'VAST::ExError'
        #'VAST::LookupTable'
        #'VAST::OSComposite'
        #'VAST::OSDialogBox'
        #'VAST::SubApplication'
        #'VAST::System'
        #'VAST::VastSignal'
        VW5PackageExporter
        WidgetWrapper
        Win32Shell
        #'stx_libcompat'
        BaselineOf
        DataStream
        #'Dolphin::FileOpenDialog'
        #'Dolphin::FileSaveDialog'
        #'Dolphin::SharedIdentitySet'
        LookPreferences
        MacLookPolicy
        #'Squeak::ResumeDebugAction'
        #'V::DynalinkUnderTest'
        #'VAST::Application'
        Win3LookPolicy
        ReferenceStream
        #'VAST::Kernel'
        (MorphicUIManager autoload)
        (SystemNavigation autoload)
    )
!

extensionMethodNames
    "lists the extension methods which are to be included in the project.
     Entries are 2-element array literals, consisting of class-name and selector.
     A correponding method with real names must be present in my concrete subclasses
     if it has extensions."

    ^ #(
        Behavior allSelectorsInProtocol:
        Behavior defaultNameStemForInstances
        Behavior kindOfSubclass
        Behavior setTraitComposition:
        Character asUnicode
        Character charCode
        Character isSafeForHTTP
        CharacterArray asComposedText
        CharacterArray asFileReference
        CharacterArray asLogicalFileSpecification
        CharacterArray asOneByteString
        CharacterArray asQualifiedReference
        CharacterArray asUrl
        CharacterArray asWideString
        CharacterArray capitalized
        CharacterArray caseInsensitiveLessOrEqual:
        CharacterArray caseSensitiveLessOrEqual:
        CharacterArray charactersExactlyMatching:
        CharacterArray convertToEncoding:
        CharacterArray displayProgressAt:from:to:during:
        CharacterArray findTokens:keep:
        CharacterArray format:
        CharacterArray intervalFromStartLine:startColumn:toEndLine:endColumn:
        CharacterArray intervalOfLine:
        CharacterArray intervalOfLineCorrespondingToIndex:
        CharacterArray lineIndicesDo:
        CharacterArray lines
        CharacterArray openInWorkspaceWithTitle:
        CharacterArray substrings:
        CharacterArray translated
        CharacterArray unescapePercents
        CharacterArray unescapePercentsWithTextEncoding:
        CharacterArray withNoLineLongerThan:
        CharacterArray withSqueakLineEndings
        Class bindingOf:
        Class objectForDataStream:
        Class storeDataOn:
        Class ultimateSourceCodeAt:ifAbsent:
        ClassDescription categoriesFor:
        ClassDescription categoriesForClass
        ClassDescription commentStamp:prior:
        ClassDescription compile:classified:withStamp:
        ClassDescription compile:classified:withStamp:notifying:logSource:
        ClassDescription compileSilently:classified:
        ClassDescription variableByteSubclass:classVariableNames:poolDictionaries:category:
        ClassOrganizer commentStamp
        Collection collectAsSet:
        Collection deepFlatten
        Collection deepFlattenInto:
        Collection detect:ifOne:
        Collection detect:ifOne:ifNone:
        Collection do:displayingProgress:
        Collection equalsTo:
        Collection flatCollect:
        Collection flatCollect:as:
        Collection flatCollectAsSet:
        Collection flatten
        Collection groupedBy:
        Collection groupedBy:affect:
        Collection includesAllOf:
        Collection includesAnyOf:
        Collection intersection:
        Collection nilSafeGroupedBy:
        Collection selectAsSet:
        Collection sorted:
        Collection symmetricDifference:
        Color newTileMorphRepresentative
        GenericException signal:
        Integer #'&'
        Integer asLargerPowerOfTwo
        Integer asSmallerPowerOfTwo
        Metaclass isTrait
        Method numTemps
        Method timeStamp
        Number asDuration
        Number newTileMorphRepresentative
        Number stringForReadout
        Object becomeForward:
        Object becomeForward:copyHash:
        Object caseError
        Object caseOf:
        Object caseOf:otherwise:
        Object comeFullyUpOnReload:
        Object deepFlattenInto:
        Object deprecated:
        Object explore
        Object implementedBySubclass
        Object inform:
        Object invalidMessage
        Object isInMemory
        Object objectForDataStream:
        Object oneWayBecome:
        Object perform:withEnoughArguments:
        Object readDataFrom:size:
        Object stbFixup:at:
        Object storeDataOn:
        Object stringForReadout
        Object stringRepresentation
        Object veryDeepCopy
        OrderedCollection overlappingPairsCollect:
        Point asLargerPowerOfTwo
        Point asSmallerPowerOfTwo
        Process isTerminated
        Process queueInterrupt:
        ReadStream nextInto:
        SequenceableCollection asTraitComposition
        SequenceableCollection pairsDistanceFrom:
        SequenceableCollection pairsSimilarityWith:
        SequenceableCollection piecesCutWhere:do:
        SequenceableCollection removePrefix:
        SequenceableCollection removeSuffix:
        SequenceableCollection runsFailing:
        SequenceableCollection runsSatisfying:
        SequenceableCollection runsSatisfying:do:
        SequenceableCollection shuffle
        SequenceableCollection shuffle:
        SequenceableCollection sliceFrom:
        SequenceableCollection sliceFrom:to:
        SequenceableCollection sliceTo:
        Set flatCollect:
        Socket bindTo:port:
        Socket localAddress
        Socket primSocketLocalPort:
        Socket receiveUDPDataInto:
        Socket sendUDPData:toHost:port:
        Socket setOption:value:
        Socket socketHandle
        Socket waitForData
        Socket waitForDataForMsecs:ifClosed:ifTimedOut:
        Socket waitForDataIfClosed:
        Stream nextInt32
        Stream nextInt32Put:
        Stream nextLittleEndianNumber:
        Stream nextLittleEndianNumber:put:
        Stream nextPutSqueakString:
        Stream nextSqueakString
        Stream nextSqueakStringOld
        Stream nextString
        Stream nextStringOld
        Stream nextStringPut:
        Stream nextWord
        String asByteString
        String asEnglishPlural
        String asGUID
        String deepFlattenInto:
        String piecesCutWhere:
        String piecesCutWhereCamelCase
        String replaceSuffix:with:
        String withInternetLineEndings
        ZipArchive contentsOf:
        ZipArchive testUTF8
        'ChangeSet class' uniqueNameLike:
        'Character class' escape
        'Collection class' ofSize:
        'FileStream class' readonlyFileNamed:do:
        'GenericException class' signal:
        'ProjectDefinition class' definitionClassForMonticelloPackage:
        'ProjectDefinition class' definitionClassForMonticelloPackage:createIfAbsent:
        'Smalltalk class' allClassesAndTraits
        'Smalltalk class' allClassesAndTraitsDo:
        'Smalltalk class' allTraits
        'Smalltalk class' globals
        'Smalltalk class' logChange:
        'Smalltalk class' os
        'Smalltalk class' registerExternalObject:
        'Smalltalk class' renameClassNamed:as:
        'Smalltalk class' unregisterExternalObject:
        'Smalltalk class' windowSystemName
        'Symbol class' allSymbols
        'Symbol class' tableSize
        CharacterArray withLineEndings:
        CharacterArray withUnixLineEndings
        ExternalAddress asInteger
        ExternalAddress fromInteger:
        ExternalAddress getHandle
        ExternalAddress handle
        ExternalLibrary ffiCall:
        'Dictionary class' newFromKeys:andValues:
        'Dictionary class' newFromPairs:
        'ExternalAddress class' allocate:
        'ExternalAddress class' gcallocate:
        'ExternalAddress class' wordSize
        'ExternalLibraryFunction class' ffiTypeSymbolFromUFFISpec:
        'ExternalLibraryFunction class' uffi:
        Collection sorted
        Dictionary at:ifPresent:ifAbsent:
        Filename asFileReference
        GenericException signal
        Integer alignedTo:
        Integer hex
        Object errorSubscriptBounds:
        Object flag:
        Object trace:
        Object traceCr
        Object traceCr:
        OrderedCollection removeDuplicates
        'Array class' with:withAll:
        'Collection class' with:withAll:
        Collection flattened
        Integer radix:
        Object isPackageManifest
        Filename readStreamDo:
        Filename writeStreamDo:
        'AbstractOperatingSystem class' runCommand:
        OSProcess isComplete
        OSProcess isRunning
        'OSProcess class' command:
        Block assert
        Block doWhileTrue:
        Filename pathString
    )
! !

!stx_libcompat class methodsFor:'description - project information'!

companyName
    "Return a companyname which will appear in <lib>.rc"

    ^ 'Claus Gittinger / eXept Software AG'

    "Modified: / 18-11-2016 / 11:48:26 / cg"
!

description
    "Return a description string which will appear in nt.def / bc.def"

    ^ 'Smalltalk/X Compatibility'
!

legalCopyright
    "Return a copyright string which will appear in <lib>.rc"

    ^ 'Copyright Claus Gittinger 1988\nCopyright eXept Software AG 2012'

    "Modified: / 18-11-2016 / 12:17:56 / cg"
!

productName
    "Return a product name which will appear in <lib>.rc"

    ^ 'Smalltalk/X'
! !

!stx_libcompat class methodsFor:'description - svn'!

svnRevisionNr
    "Return a SVN revision number of myself.
     This number is updated after a commit"

    ^ "$SVN-Revision:"'exported'"$"
! !

!stx_libcompat class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
!

version_HG
    ^ '$Changeset: <not expanded> $'
!

version_SVN
    ^ '$Id$'
! !

